function animateWalker(par,x,numsteps,outputflag,outname)
% animateWalker(par,x,numsteps,outputflag,outname)) 
%   animates the 2-d passive walking simulation
%   x should contain one full step of the states, [q1 q2 u1 u2] arranged
%   in rows.  numsteps is the # of steps to walk (the states in x are
%   repeated automatically for numsteps > 1).  An alternative way to
%   call animateawalk2 is if x contains multiple steps, then numsteps
%   can be a vector containing the starting indices for each of the
%   multiple steps.
%   To have the program save each frame as an adobe illustrator file,
%   use animateawalk2(x, numsteps, 1)

% Art Kuo Dynamic Walking

L = par.L;
R = par.R;

debg = 0;

if isempty(L), L = 1; end; % if L and R aren't defined, then choose
if isempty(R), R = 0.3; end; % values of 1 and 0.3

footn = 10; % number of pieces to break foot into
% alpha = max(abs([x(:,1); x(:,2)]))
alpha = 0.5;

xlen = length(x);

if nargin < 3, numsteps = []; end;

if length(numsteps) > 1  % numsteps is a list of step lengths
  steplist = numsteps;
  numsteps = length(numsteps);
  endindex = cumsum(steplist);
  startindex = [1 endindex-1];
else                     % numsteps is just a scalar # of steps
  if nargin < 2 | isempty(numsteps)
    numsteps = 2;        % default 2 if unspecified in input
  end
  % now convert it into a list of step lengths
  steplist = [xlen repmat(xlen-1, 1, numsteps-1)];
  endindex = repmat(xlen, numsteps, 1);
  %startindex = [1; repmat(2, numsteps-1, 1)];
  startindex = repmat(1,numsteps,1); % extra frame
end
% Now numsteps contains the number of steps, steplist
% contains the number of frames in each step, and
% startindex and endindex contain indices for each step

if nargin < 4 | isempty(outputflag)
  outputflag = 0;
end

% Estimate range of walking
distance = 2*numsteps*R*x(1,1)+(numsteps+1)*((L-R)*abs(sin(x(1,1))-sin(x(1,2))));
xlimit = [-R distance+R]-(L-R)*abs(sin(x(1,1))-sin(x(1,2)));
xlimit = [-1 3]; % fixed for easier comparison
ylimit = [-0.05 1.05];

aang = pi/6; scale = 0.02; scale2 = 2; vx2 = 0.4; vy2 = 1.2;

% A foot

% foot starts at -sin(a),cos(a)
% and goes to sin(a),cos(a)
footang = linspace(-alpha, alpha, footn);
footxy = R*[sin(footang); -cos(footang)];

% Initialize
clf; 
th1 = x(1,1); th2 = x(1,2); u1 = x(1,3);
contactpoint = -th1*R;
Rot1 = [cos(th1) -sin(th1); sin(th1) cos(th1)];
Rot2 = [cos(th2) -sin(th2); sin(th2) cos(th2)];

footx1 = Rot1(1,:)*footxy + contactpoint;
footy1 = Rot1(2,:)*footxy + R;

legsxy = [0  -sin(th1)  -sin(th1)+sin(th2);
  0   cos(th1)   cos(th1)-cos(th2)];
legsx = legsxy(1,:) + contactpoint + R*sin(th1);
legsy = legsxy(2,:) + R - R*cos(th1);       
footx2 = Rot2(1,:)*footxy + legsx(3) - R*sin(th2);
footy2 = Rot2(2,:)*footxy + legsy(3) + R*cos(th2);
pcm = legsxy(:,2) + [contactpoint+R*sin(th1);R-R*cos(th1)];
vcm = [-u1*(R + (L-R)*cos(th1)); -u1*(L-R)*sin(th1)];
velang = atan2(vcm(2),vcm(1));
velx = [0 vcm(1) vcm(1)-scale*cos(velang+aang) NaN vcm(1) vcm(1)-scale*cos(velang-aang)]+pcm(1);
vely = [0 vcm(2) vcm(2)-scale*sin(velang+aang) NaN vcm(2) vcm(2)-scale*sin(velang-aang)]+pcm(2);
velx2 = scale2*[0 vcm(1) vcm(1)-scale*cos(velang+aang) NaN vcm(1) vcm(1)-scale*cos(velang-aang)]+vx2;
vely2 = scale2*[0 vcm(2) vcm(2)-scale*sin(velang+aang) NaN vcm(2) vcm(2)-scale*sin(velang-aang)]+vy2;

set(gcf, 'DoubleBuffer', 'on');
set(gcf, 'color', [1 1 1]);
set(gca,'DataAspectRatio',[1 1 1],'Visible','off','NextPlot','Add','XLim',xlimit,'YLim',ylimit);

hf1 = line(footx1,footy1,'LineWidth',3,'EraseMode','normal'); 
hf2 = line(footx2,footy2,'LineWidth',3,'EraseMode','normal');
hlegs = line(legsx,legsy,'LineWidth',3,'EraseMode','background');
% hvel = line(velx,vely,'color','m','LineWidth',2,'EraseMode','background');
hpelv = plot(legsx(2),legsy(2),'.','MarkerSize',30,'EraseMode','background');
hgnd = line(xlimit,[0 0]-.01,'color',[0 0 0],'linewidth',2);
%hvel2 = line(velx2,vely2,'color','m','LineWidth',2,'EraseMode','background');


if outputflag & outputflag == 'm'
%     aviobj = avifile(outname);
else
    drawnow;
    % pause;
end

th1old = th1; cntr = 1;
frameNum = 1;
for j = 1:numsteps
  for i = startindex(j):endindex(j)
    th1 = x(i,1); th2 = x(i,2);
    contactpoint = contactpoint - (th1-th1old)*R; % roll forward a little
    th1old = th1;
    Rot1 = [cos(th1) -sin(th1); sin(th1) cos(th1)];
    Rot2 = [cos(th2) -sin(th2); sin(th2) cos(th2)];
    
    footx1 = Rot1(1,:)*footxy + contactpoint; footy1 = Rot1(2,:)*footxy + R;
    legsxy = [0  -sin(th1)  -sin(th1)+sin(th2);
              0   cos(th1)   cos(th1)-cos(th2)];
    legsx = legsxy(1,:) + contactpoint + R*sin(th1);
    legsy = legsxy(2,:) + R - R*cos(th1);       
    footx2 = Rot2(1,:)*footxy + legsx(3) - R*sin(th2);
    footy2 = Rot2(2,:)*footxy + legsy(3) + R*cos(th2);
    
    pcm = legsxy(:,2) + [contactpoint+R*sin(th1);R-R*cos(th1)];
    vcm = [-u1*(R + (L-R)*cos(th1)); -u1*(L-R)*sin(th1)];
    velang = atan2(vcm(2),vcm(1));
    velx = [0 vcm(1) vcm(1)-scale*cos(velang+aang) NaN vcm(1) vcm(1)-scale*cos(velang-aang)]+pcm(1);
    vely = [0 vcm(2) vcm(2)-scale*sin(velang+aang) NaN vcm(2) vcm(2)-scale*sin(velang-aang)]+pcm(2);
    velx2 = scale2*[0 vcm(1) vcm(1)-scale*cos(velang+aang) NaN vcm(1) vcm(1)-scale*cos(velang-aang)]+vx2;
    vely2 = scale2*[0 vcm(2) vcm(2)-scale*sin(velang+aang) NaN vcm(2) vcm(2)-scale*sin(velang-aang)]+vy2;
    
    set(hf1,'Xdata',footx1,'Ydata',footy1);
    set(hf2,'Xdata',footx2,'Ydata',footy2);
    set(hlegs,'Xdata',legsx,'Ydata',legsy);
%     set(hvel,'Xdata',velx,'Ydata',vely);
    set(hpelv,'Xdata',legsx(2),'Ydata',legsy(2));
    %set(hvel2,'Xdata',velx2,'Ydata',vely2);
    if 0
    if i==1 & j > 1  % stick velocity arrow
      hveli=line(velx2,vely2,'color','m','LineWidth',2,'erasemode','none');
      oldx = get(hvelo,'xdata'); oldy = get(hvelo,'ydata');
      hsang = atan2(vely2(2)-oldy(2),velx2(2)-oldx(2));
      velxh = [oldx(2) velx2(2) velx2(2)-scale2*scale*cos(hsang+aang) NaN velx2(2) velx2(2)-scale2*scale*cos(hsang-aang)];
    	velyh = [oldy(2) vely2(2) vely2(2)-scale2*scale*sin(hsang+aang) NaN vely2(2) vely2(2)-scale2*scale*sin(hsang-aang)];  
  		hvelhs = line(velxh,velyh,'color','r','Linewidth',2,'erasemode','none');    
    end
    end
    drawnow;
    if outputflag && outputflag == 'm'
        MovieOut(frameNum) = getframe(gca);
%         aviobj = addframe(aviobj,getframe(gca));
    else
        pause(1/30);
    end
    frameNum = frameNum + 1;
    if outputflag && outputflag == 'a'
      print('-dill',sprintf('walk%02d',cntr));
    end
    if debg, pause, end;
    cntr = cntr + 1;
  end
  contactpoint = contactpoint - (L-R)*(sin(th1)-sin(th2)); th1old = th2;
	%hvelo = line(velx2,vely2,'color','m','LineWidth',2);
end
if outputflag && outputflag == 'm'
    movie2avi(MovieOut,outname,'fps',29.970);
%     aviobj = close(aviobj);
end

% Have to deal with two situations:
% one step that is repeated many times
% or multiple steps that don't have to be identical
