function [xnext,tcontact,t,x] = simulateAWalker(t0, x0, par, flag)
% [xnext,tcontact,t,x] = simulateAWalker(t0, x0, par, flag)
%
% Simulate the anthropomorphic walker for a single stride or a limited amount of
% time.  At the end of the simulation (regardless of whether heel strike is
% detected) a collision is simulated.
%
% Pass 'a' for flag to get samples spaced at 30Hz.

    if nargin < 3
      flag = [];
    end
    if (length(t0) > 1)
        interval = t0;
    else
        interval = [t0 t0 + 5];
    end
    options = odeset('events', @(t,x) eventawalk2(t,x,par));
    [t,x] = ode45(@(t,x) fwalk(t,x,par), interval, x0, options);

    xnext = heelstrike(x(end,:)',par);
    tcontact = t(end);


    if flag == 'a' % for animation, do equal time steps
        t = t(1):1/30:t(end);
        [t,x] = ode45(@(t,x) fwalk(t,x,par), t, x0);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xdot = fwalk(t,x,par)
    L = par.L;
    gamma = par.gamma; 
    I = par.I;
    I1 = par.I1;
    C = par.C;
    C1 = par.C1;
    m = par.m;
    m1 = par.m1;
    R = par.R;
    g = par.g;

    k = par.k;

    phi1 = x(1);
    phi2 = x(2);
    phi1d = x(3);
    phi2d = x(4);

    T = [ -R-(C1-R)*cos(phi1),                  0
            -(C1-R)*sin(phi1),                  0
                           1,                  0
          -R-(L-R)*cos(phi1),    (L-C)*cos(phi2)
            -(L-R)*sin(phi1),    (L-C)*sin(phi2)
                           0,                  1];

    T2 = [                    (C1-R)*sin(phi1)*phi1d^2
                             -(C1-R)*cos(phi1)*phi1d^2
                                                    0
      (L-R)*sin(phi1)*phi1d^2-(L-C)*sin(phi2)*phi2d^2
     -(L-R)*cos(phi1)*phi1d^2+(L-C)*cos(phi2)*phi2d^2
                                                    0];

    M = diag([m1, m1, I1, m, m, I]);

    % vector with external forces
    f = [ m1*g*sin(gamma)
         -m1*g*cos(gamma)
          0
          m*g*sin(gamma)
         -m*g*cos(gamma)
          0];

    % spring forces
    s = (k*(phi1-phi2)/2)*[ -1; 1 ];

    % creation of reduced mass matrix
    Mr = T.'*M*T; 
    rhs = s + T.'*f  - T.'*M*T2;


    udot = Mr\rhs;

    xdot = [x(3); x(4); udot];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [value, isterminal, direction] = eventawalk2(t, x, par)
    q1 = x(1); q2 = x(2); u1 = x(3); u2 = x(4);

    value = cos(q1) - cos(q2);
    isterminal = q1<-0.001;
    direction = -1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xnew = heelstrike(xminus,par)
    L = par.L;
    gamma = par.gamma; 
    I = par.I;
    I1 = par.I1;
    C = par.C;
    C1 = par.C1;
    m = par.m;
    m1 = par.m1;
    R = par.R;
    g = par.g;
    imp = par.imp;

    phi1 = xminus(1);
    phi2 = xminus(2);
    phi1d = xminus(3);
    phi2d = xminus(4);

    qd = [phi1d; phi2d];

    M = diag([m1, m1, I1, m, m, I]);

    Tm = [ -R-(C1-R)*cos(phi1),                  0
               -(C1-R)*sin(phi1),                  0
                              1,                  0
             -R-(L-R)*cos(phi1),    (L-C)*cos(phi2)
               -(L-R)*sin(phi1),    (L-C)*sin(phi2)
                              0,                  1]; 


    Tp = [    (L-C)*cos(phi1), -R-(L-R)*cos(phi2)
                 (L-C)*sin(phi1),   -(L-R)*sin(phi2)
                               1,                  0
                               0, -R-(C1-R)*cos(phi2)
                               0,   -(C1-R)*sin(phi2)
                               0,                  1];

    impulse = [-imp*sin(phi1) imp*cos(phi1) 0 0 0 0]';

    % reduced mass matrix 
    Mr = Tp.'*M*Tp;

    rhs =  Tp.'*(M*Tm*qd + impulse);

    % solve impact equations
    solution = Mr\rhs;
    % the first four elements of solution are the velocities after impact (xh, yh, phi1d, phi2d)
    % the last two elements are the contact impulses (horizontal, vertical)
    xnew = [xminus(2); xminus(1); solution(2); solution(1)];
    % note legs are switched here

end
