function nonZeroIs = printSymMatrix(fid,name,mat,pretty,ckSparse,nTabs,nonZeroIs)
% printSymMatrix(fid,name,mat,pretty,ckSparse)
%
% Print out a symbolic matrix in a matlab usable form.
%
% pretty - try to make the output pretty? (default = false)
% ckSparse - threshold for making the output a sparse matrix. (default = 0)
%            0 = never make it sparse
%            1 = always make it sparse
% nTabs - number of tabs to preface each line with. (default = 1)
% nonZeroIs - an optional list of non-zero linear indexes to speed up
%             sparsity checking and sparse output.
    if (nargin < 4)
        pretty = 0;
    end
    if (nargin < 5)
        ckSparse = 0;
    end
    if (nargin < 6)
        nTabs = 1;
    end
    prefix = repmat(sprintf('\t'),[1,nTabs]);
    
    n = numel(mat);

    if ckSparse > 0
        if nargin < 7
            if isa(mat,'sparsesym')
                nonZeroIs = find(mat);
            else
                nonZeroIs = find(mat ~= 0);
            end
        end
        nZero = numel(mat)-length(nonZeroIs);

        if nZero/n > 1-ckSparse;
            % Matlab currently doesn't support Nd sparse matrices
            if ndims(mat) < 3 && n > 50
                mkSparse = 1;
            else
                mkSparse = -1;
            end
        else
            mkSparse = 0;
        end
    else
        mkSparse = 0;
    end

    if ~mkSparse
        fprintf(fid,'%s%s = ',prefix,name);
        if (n == 0)
            fprintf(fid,'[];\n');
        elseif (n == 1)
            fprintf(fid,'%s;\n',ent2str(mat(1)));
        elseif (ndims(mat) == 2)
            fprintf(fid,'[\n');

            if pretty
                maxLens = zeros([size(mat,2),1]);
                for i = 1:size(mat,1)
                    for j = 1:size(mat,2)
                        curStr = ent2str(mat(i,j));
                        curLen = length(curStr);
                        if curLen > maxLens(j)
                            maxLens(j) = curLen;
                        end
                    end
                end
            end

            for i = 1:size(mat,1)
                fprintf(fid,'%s\t',prefix);
                for j = 1:size(mat,2)
                    curStr = ent2str(mat(i,j));

                    if pretty
                        curLen = length(curStr);
                        padLen = 1 + maxLens(j) - curLen;
                        if (j < size(mat,2))
                            fprintf(fid,'%s%s, ', repmat(' ',[1,padLen]), curStr);
                        else
                            fprintf(fid,'%s%s', repmat(' ',[1,padLen]), curStr);
                        end
                    else
                        fprintf(fid,'%s ',[curStr, ' ']);
                    end
                end
                fprintf(fid,';\n');
            end
            fprintf(fid,'%s];\n',prefix);
        else
            matSize = size(mat);
            fprintf(fid,'zeros([%d',matSize(1));
            fprintf(fid,',%d',matSize(2:end));
            fprintf(fid,']);\n');

            for i = 1:numel(mat)
                if mat(i) ~= 0
                    curStr = ent2str(mat(i));
                    fprintf(fid,'%s%s(%d) = %s;\n',prefix,name,i,curStr);
                end
            end
        end
    else
        fprintf(fid,'%s%s = ',prefix,name);
        if (n == 0)
            fprintf(fid,'[];\n');
        else
            if mkSparse == 1
                fprintf(fid,'sparse([],[],[],%d,%d,%d);\n',size(mat,1),size(mat,2),n-nZero);
            else
                if ndims(mat) == 2
                    fprintf(fid,'zeros(%d,%d);\n',size(mat,1),size(mat,2));
                else
                    matSize = size(mat);
                    fprintf(fid,'zeros([%d',matSize(1));
                    fprintf(fid,',%d',matSize(2:end));
                    fprintf(fid,']);\n');
                end
            end

            for ii = 1:length(nonZeroIs)
                i = nonZeroIs(ii);
                curStr = ent2str(mat(i));
                fprintf(fid,'%s%s(%d) = %s;\n',prefix,name,i,curStr);
            end
        end
    end
end

function string = ent2str(ent)
    if (isa(ent,'sym'))
        string = sprintf('%s',char(ent));
%         string = strrep(string,'atan(','atan2(');
        spaces = isspace(string);
        spaceInds = find(~spaces);
        string = string(spaceInds(1):spaceInds(end));
    else
        string = sprintf('%g',ent);
    end
end
