%% Initialize paths
addpath('../quat');

%% Define initial conditions
initPosition = [0;0;0];
initOrientation = quataxisangle([1;1;0]/sqrt(2),pi/2);

initLinVel = [0;0;0];
initBodyAngVel = [1;0.5;0.25];

y0 = [ initPosition; initOrientation; initLinVel; initBodyAngVel];
t0 = 0;

opts.force = [0;0;0];
opts.bodyTorque = [0;0;0];
opts.worldTorque = [0;0;0];

opts.inertia = diag([1,10,10]);
opts.mass = 10;

nSteps = 10000;
dt = 1e-3;

%% Simulate using an Euler step
t = t0; y = y0;

ts = t;
ys = y;

for i = 1:nSteps;
    y = y + dt*rigidBodyODE(t,y,opts); % Euler step
    t = t + dt;
    y(4:7) = y(4:7)/norm(y(4:7)); % Renormalize the quaternion.
    ts = [ts t];
    ys = [ys y];
end


%% Animate the simulation
for i = 1:size(ys,2)
    figure(1);
    displayEllipsoidPart(opts.mass,diag(opts.inertia),quatrot(ys(4:7,i)),ys(1:3,i));
    xlabel('x');
    ylabel('y');
    zlabel('z');
    axis equal;
    axis([-2,2,-2,2,-2,2]);
    drawnow;
    pause(dt/2);
end

%% Compute angular momentum in world and body coordinate systems.
wAngMom = [];
bAngMom = [];
for i = 1:size(ys,2)
    q = ys(4:7,i);
    R = quatrot(q);
    
    % Body-fixed angular velocity
    bav = ys(11:13,i);
    % Body-fixed angular momentum
    bam = opts.inertia*bav;

    bAngMom = [ bAngMom bam ];

    
    % World-fixed angular velocity
    wav = R*bav;
    % World-fixed angular momentum
    wam = R*bam;
    
    wAngMom = [ wAngMom wam ];
end

figure(2);
plot(ts,bAngMom);
title('Body Angular Momentum');
legend('x','y','z');
xlabel('Time');

figure(3);
plot(ts,wAngMom);
title('World Angular Momentum');
legend('x','y','z');
xlabel('Time');


%%
nInitSteps = 100;
nSubSteps = 10;
dt = 1e-2;

qStepDeriv = false; % Use the quaternion derivative based step or not.
qStepBodyAngVel = false; % Use the body frame angular velocity or not.

angStepVel = false; % Step angular velocity or angular momentum
angStepBody = false; % Step in body or world frame

%%
p0 = initPosition;
q0 = initOrientation;
R0 = quatrot(q0);

% Compute the world frame inertia tensor
wMoI = computeWorldMoI(q0,MoI);

% Linear velocity
lv0 = initLinVel;
% Body-fixed angular velocity
bav0 = initAngVel;
% Body-fixed angular momentum
bam0 = MoI*bav0;
% World-fixed angular velocity
wav0 = R0*bav0;
% World-fixed angular momentum
wam0 = R0*MoI*bav0;

ps = p0;
qs = q0;
lvs = lv0;
bavs = bav0;
bams = bam0;
wavs = wav0;
wams = wam0;
Ks = computeKineticEnergy(bav0,MoI);

for i = 1:nInitSteps
    for j = 1:nSubSteps
        % Compute derivatives of linear and angular velocity
        linAcc = (1/mass)*force;
        totalWorldTorque = worldTorque + bodyToWorldTorque(q0,bodyTorque);
        totalBodyTorque = bodyTorque + worldToBodyTorque(q0,worldTorque);
        dwam0 = computeDerivWorldAngMom(totalWorldTorque);
        dwav0 = computeDerivWorldAngVel(q0,wav0,MoI,totalWorldTorque);
        dbam0 = computeDerivBodyAngMom(bav0,MoI,totalBodyTorque);
        dbav0 = computeDerivBodyAngVel(bav0,MoI,totalBodyTorque);

        % Integrate linear position
        p0 = p0 + (dt/nSubSteps)*lv0;

        % Integrate orientation
        if qStepDeriv
            if qStepBodyAngVel
                % Using quaternion derivative and body frame angular velocity
                q0 = q0 + (dt/nSubSteps)*computeQuatDeriv(q0,bav0,false);
            else
                % Using quaternion derivative and world frame angular velocity
                q0 = q0 + (dt/nSubSteps)*computeQuatDeriv(q0,wav0,true);
            end
        else
            if qStepBodyAngVel
                % Using quaternion multiplication and body frame angular velocity
                q0 = quatmult(q0,exp2quat((dt/nSubSteps)*bav0));
            else
                % Using quaternion multiplication and world frame angular velocity
                q0 = quatmult(exp2quat((dt/nSubSteps)*wav0),q0);
            end
        end
        q0 = q0/norm(q0); % Renormalize, just in case

        R0 = quatrot(q0);
        % Compute the world frame inertia tensor
        wMoI = computeWorldMoI(q0,MoI);

        % Integrate the linear velocity
        lv0 = lv0 + (dt/nSubSteps)*linAcc;

        if angStepVel
            if angStepBody
                % Integrate the body-fixed angular velocity
                bav0 = bav0 + (dt/nSubSteps)*dbav0;
                wav0 = R0*bav0;
                bam0 = MoI*bav0;
                wam0 = wMoI*wav0;
            else
                % Integrate the world-fixed angular velocity
                wav0 = wav0 + (dt/nSubSteps)*dwav0;
                bav0 = R0.'*wav0;
                bam0 = MoI*bav0;
                wam0 = wMoI*wav0;
            end
        else
            if angStepBody
                % Integrate the body-fixed angular momentum
                bam0 = bam0 + (dt/nSubSteps)*dbam0;
                bav0 = MoI\bam0;
                wav0 = R0*bav0;
                wam0 = wMoI*wav0;
            else
                % Integrate the world-fixed angular momentum
                wam0 = wam0 + (dt/nSubSteps)*dwam0;
                wav0 = wMoI\wam0;
                bav0 = R0.'*wav0;
                bam0 = MoI*bav0;
            end
        end
    end

    ps = [ps p0];
    qs = [qs q0];
    lvs = [lvs lv0];
    bavs = [bavs bav0];
    bams = [bams bam0];
    wavs = [wavs wav0];
    wams = [wams wam0];
    Ks = [Ks computeKineticEnergy(bav0,MoI)];
end

%%
figure(3);
subplot(2,1,1);
plot(wams.');
legend('x','y','z');

subplot(2,1,2);
plot(Ks);
legend('Rotational Energy');

%%
for i = 1:size(ps,2)
    figure(1);
    displayEllipsoidPart(mass,diag(MoI),quatrot(qs(:,i)),ps(:,i));
    xlabel('x');
    ylabel('y');
    zlabel('z');
    axis equal;
    axis([-2,2,-2,2,-2,2]);
    drawnow;
    pause(dt);
end
